@@ -26,22 +26,42 @@ my_boost::atomic_uint64_t Abstract_crawler::next_chain_id;
 
 Abstract_crawler::Abstract_crawler(
   Mysql::I_callable<bool, const Mysql::Tools::Base::Message_data&>*
-    message_handler, Simple_id_generator* object_id_generator)
-  : Abstract_chain_element(message_handler, object_id_generator)
+    message_handler, Simple_id_generator* object_id_generator,
+    Mysql::Tools::Base::Abstract_program* program)
+  : Abstract_chain_element(message_handler, object_id_generator),
+    m_program(program)
 {}
 
+Abstract_crawler::~Abstract_crawler()
+{
+  for (std::vector<I_dump_task*>::iterator it= m_dump_tasks_created.begin();
+    it != m_dump_tasks_created.end(); ++it)
+  {
+    delete *it;
+  }
+}
+
 void Abstract_crawler::register_chain_maker(I_chain_maker* new_chain_maker)
 {
   m_chain_makers.push_back(new_chain_maker);
 }
 
+Mysql::Tools::Base::Abstract_program* Abstract_crawler::get_program()
+{
+  return m_program;
+}
+
 void Abstract_crawler::process_dump_task(I_dump_task* new_dump_task)
 {
+  /* in case of error stop all further processing */
+  if (get_program()->get_error_code())
+    return;
+
   m_dump_tasks_created.push_back(new_dump_task);
 
   Item_processing_data* main_item_processing_data=
     this->new_task_created(new_dump_task);
-  bool delete_me= true;
+
   this->object_processing_starts(main_item_processing_data);
 
   for (std::vector<I_chain_maker*>::iterator it= m_chain_makers.begin();
@@ -53,7 +73,6 @@ void Abstract_crawler::process_dump_task(I_dump_task* new_dump_task)
     I_object_reader* chain= (*it)->create_chain(chain_data, new_dump_task);
     if (chain != NULL)
     {
-      delete_me= false;
       main_item_processing_data->set_chain(chain_data);
       chain->read_object(
         this->new_chain_created(
@@ -65,8 +84,6 @@ void Abstract_crawler::process_dump_task(I_dump_task* new_dump_task)
     }
   }
   this->object_processing_ends(main_item_processing_data);
-  if (delete_me)
-    delete main_item_processing_data;
 }
 
 void Abstract_crawler::wait_for_tasks_completion()
@@ -75,15 +92,18 @@ void Abstract_crawler::wait_for_tasks_completion()
     it != m_dump_tasks_created.end(); ++it)
   {
     while ((*it)->is_completed() == false)
-      my_boost::this_thread::sleep(boost::posix_time::milliseconds(1));
-    Dump_end_dump_task* end_task= dynamic_cast<Dump_end_dump_task*>(*it);
-    if (end_task != NULL && end_task->is_completed())
     {
-      for (std::vector<I_chain_maker*>::iterator it= m_chain_makers.begin();
-         it != m_chain_makers.end(); ++it)
+      /* in case of error stop all running queues */
+      if (get_program()->get_error_code())
       {
-        delete *it;
+        for (std::vector<I_chain_maker*>::iterator it= m_chain_makers.begin();
+          it != m_chain_makers.end(); ++it)
+        {
+          (*it)->stop_queues();
+        }
+        return;
       }
+      my_boost::this_thread::sleep(boost::posix_time::milliseconds(1));
     }
   }
 }
