@@ -14,18 +14,16 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "mysys_priv.h"
+#include "my_sys.h"
 #include <my_dir.h>
 #include <m_string.h>
 #include "mysys_err.h"
-#if defined(HAVE_UTIME_H)
+#include "my_thread_local.h"
+
+#ifndef _WIN32
 #include <utime.h>
-#elif defined(HAVE_SYS_UTIME_H)
+#else
 #include <sys/utime.h>
-#elif !defined(HPUX10)
-struct utimbuf {
-  time_t actime;
-  time_t modtime;
-};
 #endif
 
 	/*
@@ -60,7 +58,7 @@ int my_redel(const char *org_name, const char *tmp_name, myf MyFlags)
     char ext[20];
     ext[0]='-';
     get_date(ext+1,2+4,(time_t) 0);
-    strmov(strend(ext),REDEL_EXT);
+    my_stpcpy(strend(ext),REDEL_EXT);
     if (my_rename(org_name, fn_format(name_buff, org_name, "", ext, 2),
 		  MyFlags))
       goto end;
@@ -92,35 +90,35 @@ int my_copystat(const char *from, const char *to, int MyFlags)
   /* Copy modes */
   if (chmod(to, statbuf.st_mode & 07777))
   {
-    my_errno= errno;
+    set_my_errno(errno);
     if (MyFlags & (MY_FAE+MY_WME))
     {
       char errbuf[MYSYS_STRERROR_SIZE];
-      my_error(EE_CHANGE_PERMISSIONS, MYF(ME_BELL+ME_WAITTANG), from,
+      my_error(EE_CHANGE_PERMISSIONS, MYF(0), from,
                errno, my_strerror(errbuf, sizeof(errbuf), errno));
     }
     return -1;
   }
 
-#if !defined(__WIN__)
+#if !defined(_WIN32)
   if (statbuf.st_nlink > 1 && MyFlags & MY_LINK_WARNING)
   {
     if (MyFlags & MY_LINK_WARNING)
-      my_error(EE_LINK_WARNING,MYF(ME_BELL+ME_WAITTANG),from,statbuf.st_nlink);
+      my_error(EE_LINK_WARNING,MYF(0),from,statbuf.st_nlink);
   }
   /* Copy ownership */
   if (chown(to, statbuf.st_uid, statbuf.st_gid))
   {
-    my_errno= errno;
+    set_my_errno(errno);
     if (MyFlags & (MY_FAE+MY_WME))
     {
       char errbuf[MYSYS_STRERROR_SIZE];
-      my_error(EE_CHANGE_OWNERSHIP, MYF(ME_BELL+ME_WAITTANG), from,
+      my_error(EE_CHANGE_OWNERSHIP, MYF(0), from,
                errno, my_strerror(errbuf, sizeof(errbuf), errno));
     }
     return -1;
   }
-#endif /* !__WIN__ */
+#endif /* !_WIN32 */
 
   if (MyFlags & MY_COPYTIME)
   {
