@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -238,7 +238,7 @@ public:
   uint max_supported_keys() const;
   uint max_supported_key_parts() const;
   uint max_supported_key_length() const;
-  uint max_supported_key_part_length() const;
+  uint max_supported_key_part_length(HA_CREATE_INFO *create_info) const;
 
   virtual bool is_fk_defined_on_table_or_index(uint index);
   int get_child_or_parent_fk_list(THD *thd,
