@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -177,8 +177,7 @@ bool Sql_cmd_delete::mysql_delete(THD *thd, ha_rows limit)
   if (!using_limit && const_cond_result &&
       !(specialflag & SPECIAL_NO_NEW_FUNC) &&
       ((!thd->is_current_stmt_binlog_format_row() ||   /* not ROW binlog-format */
-        thd->is_current_stmt_binlog_disabled() || /* no binlog for this command */
-        !mysql_bin_log.is_open()) &&                   /* binary log is not opened */
+        thd->is_current_stmt_binlog_disabled()) && /* no binlog for this command */
        !(table->triggers && table->triggers->has_delete_triggers())))
   {
     /* Update the table->file->stats.records number */
@@ -662,7 +661,6 @@ bool Sql_cmd_delete::mysql_prepare_delete(THD *thd)
     List<Item>   fields;
     List<Item>   all_fields;
 
-    memset(&tables, 0, sizeof(tables));
     tables.table = table_list->table;
     tables.alias = table_list->alias;
 
@@ -757,12 +755,6 @@ int Sql_cmd_delete_multi::mysql_multi_delete_prepare(THD *thd,
   }
   *table_count= 0;
 
-  /*
-    Multi-delete can't be constructed over-union => we always have
-    single SELECT on top and have to check underlying SELECTs of it
-  */
-  select->exclude_from_table_unique_test= true;
-
   // Check the list of tables to be deleted from
   for (TABLE_LIST *delete_target= lex->auxiliary_table_list.first;
        delete_target;
@@ -799,24 +791,7 @@ int Sql_cmd_delete_multi::mysql_multi_delete_prepare(THD *thd,
 
     // Enable the following code if allowing LIMIT with multi-table DELETE
     DBUG_ASSERT(select->select_limit == 0);
-
-    /*
-      Check that table from which we delete is not used somewhere
-      inside subqueries/view.
-    */
-    TABLE_LIST *duplicate= unique_table(thd, table_ref->updatable_base_table(),
-                                        lex->query_tables, false);
-    if (duplicate)
-    {
-      update_non_unique_table_error(table_ref, "DELETE", duplicate);
-      DBUG_RETURN(true);
-    }
   }
-  /*
-    Reset the exclude flag to false so it doesn't interfare
-    with further calls to unique_table
-  */
-  select->exclude_from_table_unique_test= false;
 
   DBUG_RETURN(false);
 }
@@ -838,13 +813,43 @@ int Query_result_delete::prepare(List<Item> &values, SELECT_LEX_UNIT *u)
   DBUG_ENTER("Query_result_delete::prepare");
   unit= u;
   do_delete= true;
-  /* Don't use KEYREAD optimization on this table */
+  /*
+    Multi-delete can't be constructed over-union => we always have
+    single SELECT on top and have to check underlying SELECTs of it
+  */
+  SELECT_LEX *const select= unit->first_select();
+  select->exclude_from_table_unique_test= true;
+
   for (TABLE_LIST *walk= delete_tables; walk; walk= walk->next_local)
-    if (walk->correspondent_table)
+  {
+    if (walk->correspondent_table == NULL)
+      continue;
+
+    TABLE_LIST *ref= walk->correspondent_table->updatable_base_table();
+
+    // Don't use KEYREAD optimization on this table
+    ref->table->no_keyread= true;
+
+    /*
+      Check that table from which we delete is not used somewhere
+      inside subqueries/view.
+    */
+    TABLE_LIST *duplicate= unique_table(thd, ref,
+                                        thd->lex->query_tables, false);
+    if (duplicate)
     {
-      TABLE_LIST *ref= walk->correspondent_table->updatable_base_table();
-      ref->table->no_keyread= true;
+      update_non_unique_table_error(walk->correspondent_table,
+                                    "DELETE", duplicate);
+      DBUG_RETURN(1);
     }
+  }
+
+  /*
+    Reset the exclude flag to false so it doesn't interfer
+    with further calls to unique_table
+  */
+  select->exclude_from_table_unique_test= false;
+
   THD_STAGE_INFO(thd, stage_deleting_from_main_table);
   DBUG_RETURN(0);
 }
