@@ -1,7 +1,7 @@
 #ifndef TABLE_INCLUDED
 #define TABLE_INCLUDED
 
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -245,6 +245,13 @@ typedef struct st_order {
 */
 struct st_grant_internal_info
 {
+  st_grant_internal_info()
+    : m_schema_lookup_done(false),
+      m_schema_access(NULL),
+      m_table_lookup_done(false),
+      m_table_access(NULL)
+  {}
+
   /** True if the internal lookup by schema name was done. */
   bool m_schema_lookup_done;
   /** Cached internal schema access. */
@@ -496,10 +503,11 @@ typedef struct st_table_field_def
 class Table_check_intact
 {
 protected:
+  bool has_keys;
   virtual void report_error(uint code, const char *fmt, ...)= 0;
 
 public:
-  Table_check_intact() {}
+  Table_check_intact() : has_keys(FALSE) {}
   virtual ~Table_check_intact() {}
 
   /** Checks whether a table is intact. */
@@ -553,7 +561,7 @@ typedef I_P_List <Wait_for_flush,
 
 struct TABLE_SHARE
 {
-  TABLE_SHARE() {}                    /* Remove gcc warning */
+  TABLE_SHARE() { memset(this, 0, sizeof(*this)); }
 
   /** Category of this table. */
   TABLE_CATEGORY table_category;
@@ -979,7 +987,7 @@ typedef Bitmap<MAX_FIELDS> Field_map;
 
 struct TABLE
 {
-  TABLE() {}                               /* Remove gcc warning */
+  TABLE() { memset(this, 0, sizeof(*this)); }
   /*
     Since TABLE instances are often cleared using memset(), do not
     add virtual members and do not inherit from TABLE.
@@ -1732,7 +1740,7 @@ public:
 
 struct TABLE_LIST
 {
-  TABLE_LIST() {}                          /* Remove gcc warning */
+  TABLE_LIST() { memset(this, 0, sizeof(*this)); }
 
   /**
     Prepare TABLE_LIST that consists of one table instance to use in
@@ -1746,7 +1754,7 @@ struct TABLE_LIST
                              enum thr_lock_type lock_type_arg,
                              enum enum_mdl_type mdl_type_arg)
   {
-    memset(this, 0, sizeof(*this));
+    new (this) TABLE_LIST;
     m_map= 1;
     db= (char*) db_name_arg;
     db_length= db_length_arg;
@@ -2748,6 +2756,8 @@ struct Semijoin_mat_optimize
 */
 typedef struct st_nested_join
 {
+  st_nested_join() { memset(this, 0, sizeof(*this)); }
+
   List<TABLE_LIST>  join_list;       /* list of elements in the nested join */
   table_map         used_tables;     /* bitmap of tables in the nested join */
   table_map         not_null_tables; /* tables that rejects nulls           */
