@@ -1,4 +1,4 @@
-/* Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,7 +47,7 @@
 */
 
 int mi_assign_to_key_cache(MI_INFO *info,
-			   ulonglong key_map __attribute__((unused)),
+			   ulonglong key_map MY_ATTRIBUTE((unused)),
 			   KEY_CACHE *key_cache)
 {
   int error= 0;
