@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2005 MySQL AB & Alexey Botchkov
+/* Copyright (c) 2002, 2015, Oracle and/or its affiliates. All rights reserved.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,12 +11,9 @@
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "myisamdef.h"
-
-#ifdef HAVE_RTREE_KEYS
-
 #include "rt_index.h"
 #include "rt_key.h"
 #include "rt_mbr.h"
@@ -68,6 +65,10 @@ static double mbr_join_square(const double *a, const double *b, int n_dim)
     b += 2;
   }while (a != end);
 
+  /* Check for infinity or NaN */
+  if (my_isinf(square) || my_isnan(square))
+    square = DBL_MAX;
+
   return square;
 }
 
@@ -102,6 +103,9 @@ static void pick_seeds(SplitStruct *node, int n_entries,
   double max_d = -DBL_MAX;
   double d;
 
+  *seed_a = node;
+  *seed_b = node + 1;
+
   for (cur1 = node; cur1 < lim1; ++cur1)
   {
     for (cur2=cur1 + 1; cur2 < lim2; ++cur2)
@@ -178,18 +182,13 @@ static int split_rtree_node(SplitStruct *node, int n_entries,
                    double **d_buffer, int n_dim)
 {
   SplitStruct *cur;
-  SplitStruct *a;
-  SplitStruct *b;
+  SplitStruct *a= NULL, *b= NULL;
   double *g1 = reserve_coords(d_buffer, n_dim);
   double *g2 = reserve_coords(d_buffer, n_dim);
-  SplitStruct *next;
-  int next_node;
+  SplitStruct *next= NULL;
+  int next_node= 0;
   int i;
   SplitStruct *end = node + n_entries;
-  LINT_INIT(a);
-  LINT_INIT(b);
-  LINT_INIT(next);
-  LINT_INIT(next_node);
 
   if (all_size < min_size * 2)
   {
@@ -338,8 +337,5 @@ int rtree_split_page(MI_INFO *info, MI_KEYDEF *keyinfo, uchar *page, uchar *key,
   DBUG_PRINT("rtree", ("split new block: %lu", (ulong) *new_page_offs));
 
 split_err:
-  my_afree((uchar*) coord_buf);
   DBUG_RETURN(err_code);
 }
-
-#endif /*HAVE_RTREE_KEYS*/
