@@ -1,20 +1,13 @@
-/* Copyright (c) 2011, 2013, Oracle and/or its affiliates. All rights reserved. 
+/* Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -47,8 +40,8 @@
 */
 static inline bool operator<(const Key_use &a, const Key_use &b)
 {
-  if (a.table->tablenr != b.table->tablenr)
-    return a.table->tablenr < b.table->tablenr;
+  if (a.table_ref->tableno() != b.table_ref->tableno())
+    return a.table_ref->tableno() < b.table_ref->tableno();
   if (a.key != b.key)
     return a.key < b.key;
   if (a.keypart != b.keypart)
@@ -70,7 +63,7 @@ static inline bool operator<(const Key_use &a, const Key_use &b)
 static inline bool operator==(const Key_use &lhs, const Key_use &rhs)
 {
   return
-    lhs.table->tablenr == rhs.table->tablenr &&
+    lhs.table_ref->tableno() == rhs.table_ref->tableno() &&
     lhs.key            == rhs.key            &&
     lhs.keypart        == rhs.keypart        &&
     MY_TEST((lhs.used_tables & ~OUTER_REF_TABLE_BIT))
@@ -85,7 +78,7 @@ static inline bool operator==(const Key_use &lhs, const Key_use &rhs)
 static inline std::ostream &operator<<(std::ostream &s, const Key_use &v)
 {
   return s << "{"
-           << v.table->tablenr << ", "
+           << v.table_ref->tableno() << ", "
            << v.key            << ", "
            << v.keypart        << ", "
            << v.used_tables    << ", "
@@ -97,6 +90,8 @@ static inline std::ostream &operator<<(std::ostream &s, const Key_use &v)
 
 namespace dynarray_unittest {
 
+// We still want to unit-test this, to compare performance.
+
 /*
   Cut'n paste this function from sql_select.cc,
   to avoid linking in the entire server for this unit test.
@@ -104,8 +99,8 @@ namespace dynarray_unittest {
 inline int sort_keyuse(Key_use *a, Key_use *b)
 {
   int res;
-  if (a->table->tablenr != b->table->tablenr)
-    return (int) (a->table->tablenr - b->table->tablenr);
+  if (a->table_ref->tableno() != b->table_ref->tableno())
+    return (int) (a->table_ref->tableno() - b->table_ref->tableno());
   if (a->key != b->key)
     return (int) (a->key - b->key);
   if (a->keypart != b->keypart)
@@ -121,12 +116,12 @@ inline int sort_keyuse(Key_use *a, Key_use *b)
 
 
 // We generate some random data at startup, for testing of sorting.
-void generate_test_data(Key_use *keys, TABLE *tables, int n)
+void generate_test_data(Key_use *keys, TABLE_LIST *tables, int n)
 {
   int ix;
   for (ix= 0; ix < n; ++ix)
   {
-    tables[ix].tablenr= ix % 3;
+    tables[ix].set_tableno(ix % 3);
     keys[ix]=
       Key_use(&tables[ix],
               NULL,                           // Item      *val
@@ -167,7 +162,10 @@ public:
 
   virtual void SetUp()
   {
-    my_init_dynamic_array(&m_keyuse_dyn, sizeof(Key_use), num_elements, 64);
+    my_init_dynamic_array(&m_keyuse_dyn,
+                          PSI_NOT_INSTRUMENTED,
+                          sizeof(Key_use), NULL,
+                          num_elements, 64);
     m_keyuse_vec.reserve(num_elements);
   }
 
@@ -201,13 +199,13 @@ public:
   std::vector<Key_use>    m_keyuse_vec;
 private:
   static Key_use test_data[num_elements];
-  static TABLE   table_list[num_elements];
+  static TABLE_LIST table_list[num_elements];
 
   GTEST_DISALLOW_COPY_AND_ASSIGN_(DynArrayTest);
 };
 
 Key_use DynArrayTest::test_data[num_elements];
-TABLE   DynArrayTest::table_list[num_elements];
+TABLE_LIST DynArrayTest::table_list[num_elements];
 
 
 // Test insert_dynamic() and my_qsort().
@@ -240,13 +238,14 @@ protected:
 
   virtual void SetUp()
   {
-    init_sql_alloc(&m_mem_root, 1024, 0);
-    ASSERT_EQ(0, my_pthread_setspecific_ptr(THR_MALLOC, &m_mem_root_p));
-    MEM_ROOT *root= *my_pthread_getspecific_ptr(MEM_ROOT**, THR_MALLOC);
+    init_sql_alloc(PSI_NOT_INSTRUMENTED, &m_mem_root, 1024, 0);
+    ASSERT_EQ(0, my_set_thread_local(THR_MALLOC, &m_mem_root_p));
+    MEM_ROOT *root= *static_cast<MEM_ROOT**>(my_get_thread_local(THR_MALLOC));
     ASSERT_EQ(root, m_mem_root_p);
 
     m_array_mysys.reserve(num_elements);
     m_array_std.reserve(num_elements);
+    destroy_counter= 0;
   }
 
   virtual void TearDown()
@@ -257,14 +256,18 @@ protected:
   static void SetUpTestCase()
   {
     generate_test_data(test_data, table_list, num_elements);
-    ASSERT_EQ(0, pthread_key_create(&THR_THD, NULL));
-    ASSERT_EQ(0, pthread_key_create(&THR_MALLOC, NULL));
+    ASSERT_EQ(0, my_create_thread_local_key(&THR_THD, NULL));
+    THR_THD_initialized= true;
+    ASSERT_EQ(0, my_create_thread_local_key(&THR_MALLOC, NULL));
+    THR_MALLOC_initialized= true;
   }
 
   static void TearDownTestCase()
   {
-    pthread_key_delete(THR_THD);
-    pthread_key_delete(THR_MALLOC);
+    my_delete_thread_local_key(THR_THD);
+    THR_THD_initialized= false;
+    my_delete_thread_local_key(THR_MALLOC);
+    THR_MALLOC_initialized= false;
   }
 
   void insert_and_sort_mysys()
@@ -293,15 +296,18 @@ protected:
   MEM_ROOT *m_mem_root_p;
   Key_use_array m_array_mysys;
   Key_use_array m_array_std;
+public:
+  static size_t  destroy_counter;
 private:
   static Key_use test_data[num_elements];
-  static TABLE   table_list[num_elements];
+  static TABLE_LIST table_list[num_elements];
 
   GTEST_DISALLOW_COPY_AND_ASSIGN_(MemRootTest);
 };
 
+size_t  MemRootTest::destroy_counter;
 Key_use MemRootTest::test_data[num_elements];
-TABLE   MemRootTest::table_list[num_elements];
+TABLE_LIST MemRootTest::table_list[num_elements];
 
 
 // Test Mem_root_array::push_back() and my_qsort()
@@ -356,11 +362,26 @@ TEST_F(MemRootTest, Reserve)
 }
 
 
+// Verify that we can swap mem-root, without any leaks.
+// Run with 
+// valgrind --leak-check=full <executable> --gtest_filter='-*DeathTest*' > foo
+TEST_F(MemRootTest, CopyMemRoot)
+{
+  Mem_root_array<uint, true> intarr(m_mem_root_p);
+  // Take a copy, we do *not* free_root(own_root)
+  MEM_ROOT own_root= *m_mem_root_p;
+  intarr.set_mem_root(&own_root);
+  intarr.push_back(42);
+  *m_mem_root_p= own_root;
+}
+
+
 class DestroyCounter
 {
 public:
+  DestroyCounter() : p_counter(&MemRootTest::destroy_counter) {};
   DestroyCounter(const DestroyCounter &rhs) : p_counter(rhs.p_counter) {}
-  DestroyCounter(size_t *p) : p_counter(p) {}
+  explicit DestroyCounter(size_t *p) : p_counter(p) {}
   ~DestroyCounter() { (*p_counter)+= 1; }
 private:
   size_t *p_counter;
@@ -407,5 +428,45 @@ TEST_F(MemRootTest, ReserveDestroy)
   EXPECT_EQ(nn, counter);
 }
 
+TEST_F(MemRootTest, ResizeSame)
+{
+  Mem_root_array<DestroyCounter, false> array(m_mem_root_p);
+  array.reserve(100);
+  size_t counter= 0;
+  DestroyCounter foo(&counter);
+  for (int ix= 0; ix < 10; ++ix)
+    array.push_back(foo);
+  EXPECT_EQ(10U, array.size());
+  array.resize(10U);
+  EXPECT_EQ(10U, array.size());
+  array.clear();
+  EXPECT_EQ(10U, counter);
+}
+
+TEST_F(MemRootTest, ResizeGrow)
+{
+  Mem_root_array<DestroyCounter, false> array(m_mem_root_p);
+  array.reserve(100);
+  size_t counter= 0;
+  DestroyCounter foo(&counter);
+  array.resize(10, foo);
+  EXPECT_EQ(0U, counter);
+  array.clear();
+  EXPECT_EQ(0U, MemRootTest::destroy_counter);
+  EXPECT_EQ(10U, counter);
+}
+
+TEST_F(MemRootTest, ResizeShrink)
+{
+  Mem_root_array<DestroyCounter, false> array(m_mem_root_p);
+  array.reserve(100);
+  size_t counter= 0;
+  DestroyCounter foo(&counter);
+  array.resize(10, foo);
+  EXPECT_EQ(0U, counter);
+  array.resize(5);
+  EXPECT_EQ(5U, counter);
+}
+
 
 }
